@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2008-2016 Sergey Poznyakoff
+   Copyright (C) 2008-2018 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -139,11 +139,22 @@ parse_index_entry(const char *filename, size_t line,
     int rc;
     
     memset(&itr, 0, sizeof(itr));
+    memset(&idx, 0, sizeof(idx));
     
     utf8_iter_first(&itr, buf);
 
     rc = 0;
-    for (nfield = 0; nfield < 3; nfield++) {
+    /* A valid index file contains three to four columns per line:
+         0 - Headword.
+	 1 - Offset of the article in the dictionary file.
+	 2 - Size of the article in bytes
+	 3 - Optional original headword.
+
+      Column 0 is used for searches. Column 3, if present, is used when
+      returning the results of the search. If it is not supplied, column 0
+      is used.
+    */
+    for (nfield = 0; nfield < 4; nfield++) {
 	char *start, *end;
 	size_t len;
 	
@@ -179,6 +190,14 @@ parse_index_entry(const char *filename, size_t line,
 	    idx.word[len] = 0;
 	    idx.length = len;
 	    idx.wordlen = utf8_strlen(idx.word);
+	} else if (nfield == 3) {
+	    idx.orig = malloc(len + 1);
+	    if (!idx.orig) {
+		memerr("parse_index_entry");
+		return 1;
+	    }
+	    memcpy(idx.orig, start, len);
+	    idx.orig[len] = 0;	    
 	} else {
 	    size_t n;
 	    
@@ -224,6 +243,7 @@ free_index_entry(void *item, void *data)
 {
     struct index_entry *ep = item;
     free(ep->word);
+    free(ep->orig);
     free(ep);
     return 0;
 }
@@ -777,6 +797,13 @@ mod_descr(dico_handle_t hp)
     return ptr;
 }
 
+static char *
+mod_mime_header(dico_handle_t hp)
+{
+    struct dictdb *db = (struct dictdb *) hp;
+    return find_db_entry(db, DICTORG_ENTRY_MIME_HEADER);
+}
+
 
 static dico_result_t
 _match_simple(struct dictdb *db, entry_match_t match, const char *word)
@@ -928,9 +955,11 @@ mod_output_result (dico_result_t rp, size_t n, dico_stream_t str)
     
     
     switch (res->type) {
-    case result_match:
-	dico_stream_write(str, ep->word, strlen(ep->word));
+    case result_match: {
+	char *headword = ep->orig ? ep->orig : ep->word;
+	dico_stream_write(str, headword, strlen(headword));
 	break;
+    }
 	
     case result_define:
 	printdef(str, res->db, ep);
@@ -962,20 +991,18 @@ mod_free_result(dico_result_t rp)
 }
 
 struct dico_database_module DICO_EXPORT(dictorg, module) = {
-    DICO_MODULE_VERSION,
-    DICO_CAPA_NONE,
-    mod_init,
-    mod_init_db,
-    mod_free_db,
-    NULL,
-    NULL,
-    mod_info,
-    mod_descr,
-    NULL, /* FIXME: dico_db_lang */
-    mod_match,
-    mod_define,
-    mod_output_result,
-    mod_result_count,
-    mod_compare_count,
-    mod_free_result
+    .dico_version = DICO_MODULE_VERSION,
+    .dico_capabilities = DICO_CAPA_NONE,
+    .dico_init = mod_init,
+    .dico_init_db = mod_init_db,
+    .dico_free_db = mod_free_db,
+    .dico_db_info = mod_info,
+    .dico_db_descr = mod_descr,
+    .dico_match = mod_match,
+    .dico_define = mod_define,
+    .dico_output_result = mod_output_result,
+    .dico_result_count = mod_result_count,
+    .dico_compare_count = mod_compare_count,
+    .dico_free_result = mod_free_result,
+    .dico_db_mime_header = mod_mime_header
 };
