@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 2012-2016 Sergey Poznyakoff
+   Copyright (C) 2012-2018 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -237,8 +237,10 @@ gcide_init_db(const char *dbname, int argc, char **argv)
 	{ DICO_OPTSTR(idxdir), dico_opt_string, &idx_dir },
 	{ DICO_OPTSTR(index-program), dico_opt_string, &idxgcide },
 	{ DICO_OPTSTR(index-cache-size), dico_opt_long, &idx_cache_size },
-	{ DICO_OPTSTR(suppress-pr), dico_opt_bitmask, &flags, .v.value = GCIDE_NOPR },
-	{ DICO_OPTSTR(debug-lex), dico_opt_bitmask, &flags, .v.value = GCIDE_DBGLEX },
+	{ DICO_OPTSTR(suppress-pr), dico_opt_bitmask, &flags, 
+          .v.value = GCIDE_NOPR },
+	{ DICO_OPTSTR(debug-lex), dico_opt_bitmask, &flags, 
+          .v.value = GCIDE_DBGLEX },
 	{ NULL }
     };
     
@@ -820,29 +822,18 @@ gcide_free_result(dico_result_t rp)
     dico_list_destroy(&res->list);
 }
 
-static int
-gcide_result_headers(dico_result_t rp, dico_assoc_list_t hdr)
-{  
-    /* FIXME */
-    return 0;
-}
-
 struct dico_database_module DICO_EXPORT(gcide, module) = {
-    DICO_MODULE_VERSION,
-    DICO_CAPA_NONE,
-    gcide_init,
-    gcide_init_db,
-    gcide_free_db,
-    NULL,
-    NULL,
-    gcide_info,
-    gcide_descr,
-    NULL,
-    gcide_match,
-    gcide_define,
-    gcide_output_result,
-    gcide_result_count,
-    gcide_compare_count,
-    gcide_free_result,
-    gcide_result_headers
+    .dico_version = DICO_MODULE_VERSION,
+    .dico_capabilities = DICO_CAPA_NONE,
+    .dico_init = gcide_init,
+    .dico_init_db = gcide_init_db,
+    .dico_free_db = gcide_free_db,
+    .dico_db_info = gcide_info,
+    .dico_db_descr = gcide_descr,
+    .dico_match = gcide_match,
+    .dico_define = gcide_define,
+    .dico_output_result = gcide_output_result,
+    .dico_result_count = gcide_result_count,
+    .dico_compare_count = gcide_compare_count,
+    .dico_free_result = gcide_free_result
 };
